# This is the first associative memory that we use in the paper. 
# It does not include the beverage bumps, and all weights are static.
#

import sys

import numpy as np
import pyNN.nest as sim
import cPickle as pickle
from nealCoverClass import NealCoverFunctions
from stateMachineClass import FSAHelperFunctions
from hotCoffeeTopologyClass import HotCoffeeClass

#
COFFEE = 0
COKE = 1
COLD = 2
WARM = 3
HOT = 4

#
def createTwoInputs():
	    inputSpikeTimes0 = [10.0]
	    inputSpikeTimes1 = [50.0]
	    spikeArray0 = {'spike_times': [inputSpikeTimes0]}
	    spikeGen0=sim.Population(1,sim.SpikeSourceArray,spikeArray0,
					   label='inputSpikes_0')
	    spikeArray1 = {'spike_times': [inputSpikeTimes1]}
	    spikeGen1=sim.Population(1, sim.SpikeSourceArray, spikeArray1,
					   label='inputSpikes_1')

	    return [spikeGen0,spikeGen1]
	

#--main
args = sys.argv
Duration = 300  #this is changed for longer runs for merging streams
numberArgs = args.__len__()
if (numberArgs == 2):
    testType  = int(args[1])
else:
    testType = 12
print testType


simName = "nest"
spinnVersion = -1
neal = NealCoverFunctions(simName,sim,spinnVersion)
fsa = FSAHelperFunctions(simName,sim,neal,spinnVersion)
hotCoffeeTopology = HotCoffeeClass(simName,sim,neal,fsa,spinnVersion)

hotCoffeeTopology.makeJustBumpHotCoffeeTopology()
hotCoffeeTopology.buildHotCoffeeTopology()
hotCoffeeTopology.connectSemanticTempsToTemps()

pop = hotCoffeeTopology.getSemanticsNeurons()
inputPop = hotCoffeeTopology.getTemperatureBumpNeurons()

#TEST 
spikeSource = createTwoInputs()
if testType == 1: # (coffee)->  just coffeee
    fsa.turnOnStateFromSpikeSource(spikeSource[0],pop,COFFEE)#coffee
elif testType == 2: # (coke)->  just coke
    fsa.turnOnStateFromSpikeSource(spikeSource[0],pop,COKE)#coke
elif testType == 3: # (cold) -> just cold
    fsa.turnOnStateFromSpikeSource(spikeSource[0],pop,COLD)#cold
elif testType == 4: # (warm) -> just warm
    fsa.turnOnStateFromSpikeSource(spikeSource[0],pop,WARM)
elif testType == 5: # hot
    fsa.turnOnStateFromSpikeSource(spikeSource[0],pop,HOT)

elif testType == 6: # temperature turns on nothing
    for inputTemp in range (10,13):
        fsa.turnOnOneNeuronFromSpikeSource(spikeSource[0],inputPop,inputTemp)

elif testType == 7: # (coffee and cold) -> temp 0-24
    Duration = 5000
    fsa.turnOnStateFromSpikeSource(spikeSource[0],pop,COFFEE)#coffee
    fsa.turnOnStateFromSpikeSource(spikeSource[1],pop,COLD)#cold
elif testType == 8: # temperature & cold -> coffee
    for inputTemp in range (14,17):
        fsa.turnOnOneNeuronFromSpikeSource(spikeSource[0],inputPop,inputTemp)
    fsa.turnOnStateFromSpikeSource(spikeSource[1],pop,COLD)#cold
elif testType == 9: # coffee & temperature -> cold
    for inputTemp in range (10,13):
        fsa.turnOnOneNeuronFromSpikeSource(spikeSource[0],inputPop,inputTemp)
    fsa.turnOnStateFromSpikeSource(spikeSource[1],pop,COFFEE)#coffee

elif testType == 10: # (coffee and warm) -> temp 25-69
    Duration = 5000
    fsa.turnOnStateFromSpikeSource(spikeSource[0],pop,COFFEE)#coffee
    fsa.turnOnStateFromSpikeSource(spikeSource[1],pop,WARM)#cold
elif testType == 11: # temperature & warm -> coffee
    for inputTemp in range (36,40):
        fsa.turnOnOneNeuronFromSpikeSource(spikeSource[0],inputPop,inputTemp)
    fsa.turnOnStateFromSpikeSource(spikeSource[1],pop,WARM)#warm
elif testType == 12: # temperature & coffee -> warm
    for inputTemp in range (36,40):
        fsa.turnOnOneNeuronFromSpikeSource(spikeSource[0],inputPop,inputTemp)
    fsa.turnOnStateFromSpikeSource(spikeSource[1],pop,COFFEE)

elif testType == 13: # (coffee and hot) -> temp 80
    Duration = 5000
    fsa.turnOnStateFromSpikeSource(spikeSource[0],pop,COFFEE)#coffee
    fsa.turnOnStateFromSpikeSource(spikeSource[1],pop,HOT)
elif testType == 14: # temperature-90 & hot -> coffee
    for inputTemp in range (88,91):
        fsa.turnOnOneNeuronFromSpikeSource(spikeSource[0],inputPop,inputTemp)
    fsa.turnOnStateFromSpikeSource(spikeSource[1],pop,HOT)#hot
elif testType == 15: # temperature-90 & coffee -> hot
    for inputTemp in range (78,81):
        fsa.turnOnOneNeuronFromSpikeSource(spikeSource[0],inputPop,inputTemp)
    fsa.turnOnStateFromSpikeSource(spikeSource[1],pop,COFFEE)

elif testType == 16: # (coke and cold) -> temp 0-4
    fsa.turnOnStateFromSpikeSource(spikeSource[0],pop,COKE)
    fsa.turnOnStateFromSpikeSource(spikeSource[1],pop,COLD)#cold
elif testType == 17: # (coke and temperate-3 ) -> cold
    fsa.turnOnStateFromSpikeSource(spikeSource[0],pop,COKE)
    for inputTemp in range (2,5):
        fsa.turnOnOneNeuronFromSpikeSource(spikeSource[1],inputPop,inputTemp)
elif testType == 18: # (coke and temperate-5 ) -> coke
    fsa.turnOnStateFromSpikeSource(spikeSource[0],pop,COLD)
    for inputTemp in range (1,4):
        fsa.turnOnOneNeuronFromSpikeSource(spikeSource[1],inputPop,inputTemp)

elif testType == 19: # (coke and warm) -> temp 11
    Duration = 1000
    fsa.turnOnStateFromSpikeSource(spikeSource[0],pop,COKE)
    fsa.turnOnStateFromSpikeSource(spikeSource[1],pop,WARM)
elif testType == 20: # (coke and temperate-11 ) -> WARM
    fsa.turnOnStateFromSpikeSource(spikeSource[0],pop,COKE)
    for inputTemp in range (15,18):
        fsa.turnOnOneNeuronFromSpikeSource(spikeSource[1],inputPop,inputTemp)
elif testType == 21: # (warm and temperate-11 ) -> coke
    fsa.turnOnStateFromSpikeSource(spikeSource[0],pop,WARM)
    for inputTemp in range (14,17):
        fsa.turnOnOneNeuronFromSpikeSource(spikeSource[1],inputPop,inputTemp)

elif testType == 22: # (coke and hot) -> temp-50
    Duration = 20000
    fsa.turnOnStateFromSpikeSource(spikeSource[0],pop,COKE)
    fsa.turnOnStateFromSpikeSource(spikeSource[1],pop,HOT)
elif testType == 23: # (coke and temp-50) -> HOT
    fsa.turnOnStateFromSpikeSource(spikeSource[0],pop,COKE)
    for inputTemp in range (48,51):
        fsa.turnOnOneNeuronFromSpikeSource(spikeSource[1],inputPop,inputTemp)
elif testType == 24: # (hot and temp-50) -> COKE
    fsa.turnOnStateFromSpikeSource(spikeSource[0],pop,HOT)
    for inputTemp in range (48,51):
        fsa.turnOnOneNeuronFromSpikeSource(spikeSource[1],inputPop,inputTemp)

else: print "No testType=", testType


neal.nealApplyProjections()
sim.run(Duration)
sim.end()

hotCoffeeTopology.printTopologySpikes("paperData/bumpAM/bumpStaticAMTest")
