#
#
#
import sys

import numpy as np
import pyNN.nest as sim
import cPickle as pickle
from nealCoverClass import NealCoverFunctions
from stateMachineClass import FSAHelperFunctions
from stableBumpClass import StableBump

def init():
    sim.setup(timestep=neal.DELAY,min_delay=neal.DELAY,
                    max_delay=neal.DELAY, debug=0)

#These connectMerge defs are for the connectivity to merge two distant streams 
#to two nearby streams.
def connectMergeAll(fromPop,toPop,numNeurons):
    weight = 0.0006
    connector = []
    for fromNeuron in range(0,numNeurons):
        for toNeuron in range(0,numNeurons):
            connector = connector + [(fromNeuron,toNeuron,weight, neal.DELAY)]

    neal.nealProjection(fromPop, toPop, connector, 'excitatory')

def connectMergeInhib(inhibPop,numNeurons):
    weight = 0.03
    connector = []
    for neuron in range(0,numNeurons):
        if (neuron > 0):
            connector = connector + [(neuron,neuron-1,weight, neal.DELAY)]
        if (neuron < (numNeurons -1 )):
            connector = connector + [(neuron,neuron+1,weight, neal.DELAY)]

    #print connector
    neal.nealProjection(inhibPop, inhibPop, connector, 'excitatory')

def connectMergeOneToOne(fromPop,toPop,numNeurons):
    weight = 0.01
    connector = []
    for neuron in range(0,numNeurons):
        connector = connector + [(neuron,neuron,weight, neal.DELAY)]

    neal.nealProjection(fromPop, toPop, connector, 'excitatory')

def connectMergeInhibToBump(self,nninhib,toPop,numNeurons):
    inhibConnector = []
    for fromNeuron in range(0,numNeurons):
        for toNeuron in range(0,numNeurons):
            distance = abs(fromNeuron-toNeuron)
            
            base = 1.02
            if (distance > 15):
                distanceMod = base**(distance)
                weight = distanceMod*0.04
                #print distance,distanceMod,weight
                inhibConnector = inhibConnector + [(fromNeuron,toNeuron,weight, neal.DELAY)]
    neal.nealProjection(inhibPop, toPop, inhibConnector, 'inhibitory')
    

#--main
args = sys.argv
numberArgs = args.__len__()
if (numberArgs == 2):
    testType  = int(args[1])
else:
    testType = 2
print testType

simName = "nest"
spinnVersion = -1
neal = NealCoverFunctions(simName,sim,spinnVersion)
fsa = FSAHelperFunctions(simName,sim,neal,spinnVersion)
stableBump = StableBump(neal,fsa)

init()

neurons = 100
cellClass=sim.IF_cond_exp
cellParams=fsa.CELL_PARAMS
tempPop = stableBump.stableBump(neurons,cellClass,cellParams,2,4,0.08,0.08)
tempPop.record(['spikes','v','gsyn_exc','gsyn_inh'])

inhibPop = sim.Population(neurons,cellClass,cellParams)
inhibPop.record({'spikes','v'})

#overcomePop = sim.Population(neurons, sim.IF_cond_exp,fsa.CELL_PARAMS)
#overcomePop.record(['spikes','v','gsyn_exc','gsyn_inh'])

#
connectMergeInhib (inhibPop,neurons)
connectMergeAll(tempPop,inhibPop,neurons)
connectMergeOneToOne(tempPop,inhibPop,neurons)
connectMergeInhibToBump(stableBump,inhibPop,tempPop,neurons)

#
#connectPopToOvercome(tempPop,overcomePop)
#connectOvercomeToPop(overcomePop,tempPop)
#connectInhibOvercomeToPop(overcomePop,tempPop)

#TEST  
source0 = stableBump.createTwoInputs()
spikeSource = stableBump.createTwoInputs()
if testType == 1: 
    for inputTemp in range (10,13):
        stableBump.turnOnCellFromSpikeSource(spikeSource[0],tempPop,inputTemp)
elif testType == 2: 
    for inputTemp in range (13,31):
        stableBump.turnOnCellFromSpikeSource(spikeSource[0],tempPop,inputTemp)
elif testType == 3: 
    for inputTemp in range (0,10): #10
        stableBump.turnOnCellFromSpikeSource(spikeSource[0],tempPop,inputTemp)
elif testType == 4: 
    for inputTemp in range (10,25): #15
        stableBump.turnOnCellFromSpikeSource(spikeSource[0],tempPop,inputTemp)
elif testType == 5: 
    for inputTemp in range (0,25): #25
        stableBump.turnOnCellFromSpikeSource(spikeSource[0],tempPop,inputTemp)
elif testType == 6: 
    for inputTemp in range (70,100): #30
        stableBump.turnOnCellFromSpikeSource(spikeSource[0],tempPop,inputTemp)
elif testType == 7: 
    for inputTemp in range (25,70): #45
        stableBump.turnOnCellFromSpikeSource(spikeSource[0],tempPop,inputTemp)
elif testType == 8: 
    for inputTemp in range (25,100): #75
        stableBump.turnOnCellFromSpikeSource(spikeSource[0],tempPop,inputTemp)

else: print "No testType=", testType

neal.nealApplyProjections()
sim.run(1000)
sim.end()

tempPopPkl = tempPop.write_data("paperData/convergeTemp.pkl")
#tempPop.print_v("outputs/converge/popTempV.pkl")
#tempPop.write_data("outputs/converge/popTempE.pkl",'gsyn_exc')
#tempPop.write_data("outputs/converge/popTempI.pkl",'gsyn_inh')
inhibPopPkl = inhibPop.write_data("paperData/convergeInhib.pkl")

