# This is the simple associative memory that we use in the paper. 
#
#
#
#
import sys
import numpy as np
from stableBumpClass import StableBump
from mergeClass import MergeBump

BEVERAGE_TYPE_NO_INTERN = 1
BEVERAGE_TYPE_JUST_BUMP = 2
BEVERAGE_TYPE_FULL = 3
class HotCoffeeClass:
    def __init__(self, simName,sim,neal,fsa,spinnVersion):
        self.simName = simName
        self.sim = sim
        self.neal = neal
        self.fsa = fsa
        self.spinnVersion = spinnVersion
        self.stableBump = StableBump(self.neal,self.fsa)
        sim.setup(timestep=self.neal.DELAY,min_delay=self.neal.DELAY,
                    max_delay=self.neal.DELAY, debug=0)
        self.beverageType = BEVERAGE_TYPE_FULL

    COFFEE = 0
    COKE = 1
    COLD = 2
    WARM = 3
    HOT = 4
    numberNeurons = 100

    def makeInputTemperatureBump(self):
        self.temperaturePop=self.stableBump.stableBump(self.numberNeurons,
                self.sim.IF_cond_exp,self.fsa.CELL_PARAMS,2,4,0.08,0.08)
        self.stableBump.setupRecording(self.temperaturePop)

    def getTemperatureBumpNeurons(self):
        return self.temperaturePop

    def makeMergeBump(self):
        temperatureObject = MergeBump(self.neal,self.fsa)

        #it needs a bit of extra wt to overcome the inhibition from inhibPop
        temperatureObject.makeMergeBump(self.numberNeurons,2,4,
                    0.08,0.08,toOvercomeWt=0.0020,inhibOvercomeWt = 0.5)
        temperatureObject.setupRecording()
        return temperatureObject

    #just the bump and overcome, no merge
    def makeNoOvercomeBump(self):
        temperatureObject = MergeBump(self.neal,self.fsa)

        #it needs a bit of extra wt to overcome the inhibition from inhibPop
        temperatureObject.makeNoOvercomeBump(self.numberNeurons,2,4,
                    0.08,0.08,toOvercomeWt=0.002,inhibOvercomeWt = 0.5)
        temperatureObject.setupRecording()
        return temperatureObject

    def makeCoffeeMergeBump(self):
        self.coffeeTemperatureObject = self.makeMergeBump()

    def makeCoffeeNoOvercomeBump(self):
        self.coffeeTemperatureObject = self.makeNoOvercomeBump()

    def getCoffeeTemperatureBumpNeurons(self):
        if (self.beverageType == BEVERAGE_TYPE_NO_INTERN):
            return self.coffeeTemperatureObject
        elif (self.beverageType == BEVERAGE_TYPE_JUST_BUMP):
            return self.coffeeTemperatureObject
        return self.coffeeTemperatureObject.getStableBumpPop()

    def makeCokeMergeBump(self):
        self.cokeTemperatureObject = self.makeMergeBump()

    def makeCokeNoOvercomeBump(self):
        self.cokeTemperatureObject = self.makeNoOvercomeBump()

    def getCokeTemperatureBumpNeurons(self):
        if (self.beverageType == BEVERAGE_TYPE_NO_INTERN):
            return self.cokeTemperatureObject
        elif (self.beverageType == BEVERAGE_TYPE_JUST_BUMP):
            return self.cokeTemperatureObject
        return self.cokeTemperatureObject.getStableBumpPop()

    def makeSemanticsBinaryCAs(self):
        self.semanticsPop = self.sim.Population(50,self.sim.IF_cond_exp,
                                      self.fsa.CELL_PARAMS)
        self.fsa.makeCA(self.semanticsPop,self.COFFEE) # coffee 0
        self.fsa.makeCA(self.semanticsPop,self.COKE) # coke 1
        self.fsa.makeCA(self.semanticsPop,self.COLD) # cold 2
        self.fsa.makeCA(self.semanticsPop,self.WARM) # warm 3
        self.fsa.makeCA(self.semanticsPop,self.HOT) # hot 4

        self.semanticsPop.record('spikes')

    def getSemanticsNeurons(self):
        return self.semanticsPop

    def makeInhibitoryPop(self):
        self.inhibitoryPop =self.sim.Population(10,self.sim.IF_cond_exp,
                                                        self.fsa.CELL_PARAMS)
        self.inhibitoryPop.record('spikes')

    def getInhibitoryNeurons(self):
        return self.inhibitoryPop

    def connectInhib(self,inhibPopulation, aTemperaturePopulation):
        toInhibWt = 0.04
        fromInhibWt = 0.016
        for temp in range(0,100):
            for inhibOffset in range(0,10):
                self.fsa.oneNeuronStimulatesOneNeuron(aTemperaturePopulation,
                        temp,inhibPopulation,inhibOffset,toInhibWt)
                self.fsa.oneNeuronInhibitsOneNeuron(inhibPopulation,inhibOffset,
                    aTemperaturePopulation,temp,fromInhibWt)

    def buildHotCoffeeTopology(self):
        coffeePopulation = self.getCoffeeTemperatureBumpNeurons()
        cokePopulation = self.getCokeTemperatureBumpNeurons()
        self.connectInhib(self.inhibitoryPop,cokePopulation)
        self.connectInhib(self.inhibitoryPop,coffeePopulation)

        semToTempWt = 0.001 
        tempToDrinkWt = 0.01
    
        #input temp stimulates coffee and coke temp
        for temp in range(0,100):
            self.fsa.oneNeuronStimulatesOneNeuron(self.temperaturePop,
                                    temp,coffeePopulation,temp,tempToDrinkWt)
            self.fsa.oneNeuronStimulatesOneNeuron(self.temperaturePop,temp,
                                         cokePopulation,temp,tempToDrinkWt)
            #coffee-> coffeepop
            self.fsa.stateStimulatesOneNeuron(self.semanticsPop,self.COFFEE,
                                         coffeePopulation,temp,semToTempWt)
            #coke -> cokepop
            self.fsa.stateStimulatesOneNeuron(self.semanticsPop,self.COKE,
                                         cokePopulation,temp,semToTempWt)

    def connectSemanticTempsToTemps(self):
        coffeePopulation = self.getCoffeeTemperatureBumpNeurons()
        cokePopulation = self.getCokeTemperatureBumpNeurons()
        toTempWt = 0.0015 
        tempToSemWt = 0.025

        #coffee <-> temperatures
        for temp in range(0,25):
            #coffeePop -> cold
            self.fsa.oneNeuronStimulatesState(coffeePopulation,temp,
                            self.semanticsPop,self.COLD,tempToSemWt)
            #cold-> cofeePop
            self.fsa.stateStimulatesOneNeuron(self.semanticsPop,self.COLD,
                        coffeePopulation,temp,toTempWt)

        for temp in range(25,70): 
            #coffeePop -> warm
            self.fsa.oneNeuronStimulatesState(coffeePopulation,temp,
                self.semanticsPop,self.WARM,tempToSemWt)
            #warm-> cofeePop
            self.fsa.stateStimulatesOneNeuron(self.semanticsPop,self.WARM,
                                     coffeePopulation,temp,toTempWt)


        for temp in range(70,100):
            #coffeePop -> hot
            self.fsa.oneNeuronStimulatesState(coffeePopulation,temp,
                        self.semanticsPop,self.HOT,tempToSemWt)
            #hot-> cofeePop
            self.fsa.stateStimulatesOneNeuron(self.semanticsPop,self.HOT,
                                     coffeePopulation,temp,toTempWt)

        #coke <-> temperatures
        for temp in range(0,10):
            #cokePop -> cold  
            self.fsa.oneNeuronStimulatesState(cokePopulation,temp,
                self.semanticsPop,self.COLD,tempToSemWt)
            #cold-> cokePop
            self.fsa.stateStimulatesOneNeuron(self.semanticsPop,self.COLD,
                                     cokePopulation,temp,toTempWt)

        #coke <-> temperatures
        for temp in range(10,24):
            #cokePop -> warm
            self.fsa.oneNeuronStimulatesState(cokePopulation,temp,
                        self.semanticsPop,self.WARM,tempToSemWt)
            #warm-> cokePop
            self.fsa.stateStimulatesOneNeuron(self.semanticsPop,self.WARM,
                                     cokePopulation,temp,toTempWt)

        for temp in range(25,100):
            #cokePop -> hot
            self.fsa.oneNeuronStimulatesState(cokePopulation,temp,
                self.semanticsPop,self.HOT,tempToSemWt)
            #hot-> cokePop
            self.fsa.stateStimulatesOneNeuron(self.semanticsPop,self.HOT,
                                     cokePopulation,temp,toTempWt)

    def readSemanticTempsToTemps(self):
        coffeePopulation = self.getCoffeeTemperatureBumpNeurons()
        cokePopulation = self.getCokeTemperatureBumpNeurons()
        semToCoffeeWts = np.transpose(np.loadtxt("results/coffeeSemWts.txt")) 
        semToCokeWts = np.transpose(np.loadtxt("results/cokeSemWts.txt"))
        coffeeToSemWts = np.transpose(np.loadtxt("results/semCoffeeWts.txt"))
        cokeToSemWts = np.transpose(np.loadtxt("results/semCokeWts.txt"))
   
        #coffee 
        for cellDrink in range(0,100):
            for cellSem in (20,21,22,23,24,25,26,27,30,31,32,33,34,35,36,37,40,41,42,43,44,45,46,47):
                #coffeePop -> sem
                conn1 = self.fsa.oneNeuronStimulatesOneNeuron(coffeePopulation,
                                cellDrink,self.semanticsPop,cellSem,
                                semToCoffeeWts[cellDrink][cellSem])
                #sem -> coffeePop
                conn2 =self.fsa.oneNeuronStimulatesOneNeuron(self.semanticsPop, 
                                cellSem, coffeePopulation,cellDrink,
                                coffeeToSemWts[cellSem][cellDrink])
	        #print conn2

        #coke 
        for cellDrink in range(0,100):
            for cellSem in (20,21,22,23,24,25,26,27,30,31,32,33,34,35,36,37,40,41,42,43,44,45,46,47):
                #cokePop -> sem
                conn3 = self.fsa.oneNeuronStimulatesOneNeuron(cokePopulation,
                                cellDrink,self.semanticsPop,cellSem,
                                semToCokeWts[cellDrink][cellSem])
                #print conn3

                #sem -> cokePop
                conn4 =self.fsa.oneNeuronStimulatesOneNeuron(self.semanticsPop, 
                            cellSem, cokePopulation,cellDrink,
                            cokeToSemWts[cellSem][cellDrink])
                #print conn4

    def connectTempsToSemanticBeverages(self):
        coffeePopulation = self.getCoffeeTemperatureBumpNeurons()
        cokePopulation = self.getCokeTemperatureBumpNeurons()
        tempToSemWt = 0.025 #0.003 

        #coffee <-> temperatures
        for temp in range(0,100):
            #coffeePop -> coffee 
            self.fsa.oneNeuronStimulatesState(coffeePopulation,temp,
                                self.semanticsPop,self.COFFEE,tempToSemWt)
            #cokePop -> coke
            self.fsa.oneNeuronStimulatesState(cokePopulation,temp,
                    self.semanticsPop,self.COKE,tempToSemWt)

    def makeSimpleHotCoffeeTopology(self):
        self.beverageType = BEVERAGE_TYPE_NO_INTERN
        self.makeInputTemperatureBump()
        self.coffeeTemperatureObject = self.sim.Population(self.numberNeurons,
                        self.sim.IF_cond_exp,self.fsa.CELL_PARAMS)
        self.cokeTemperatureObject = self.sim.Population(self.numberNeurons,
                        self.sim.IF_cond_exp,self.fsa.CELL_PARAMS)
        self.coffeeTemperatureObject.record('spikes')
        self.cokeTemperatureObject.record('spikes')
        self.makeSemanticsBinaryCAs()
        self.makeInhibitoryPop()

    def makeJustBumpHotCoffeeTopology(self):
        self.beverageType = BEVERAGE_TYPE_JUST_BUMP
        self.makeInputTemperatureBump()
        self.coffeeTemperatureObject=self.stableBump.stableBump(self.numberNeurons,
                self.sim.IF_cond_exp,self.fsa.CELL_PARAMS,2,4,0.08,0.08)
        self.stableBump.setupRecording(self.coffeeTemperatureObject)
        self.cokeTemperatureObject=self.stableBump.stableBump(self.numberNeurons,
                self.sim.IF_cond_exp,self.fsa.CELL_PARAMS,2,4,0.08,0.08)
        self.stableBump.setupRecording(self.cokeTemperatureObject)
        self.makeSemanticsBinaryCAs()
        self.makeInhibitoryPop()

    def makeHotCoffeeTopology(self):
        self.makeInputTemperatureBump()
        self.makeCoffeeMergeBump()
        self.makeCokeMergeBump()
        self.makeSemanticsBinaryCAs()
        self.makeInhibitoryPop()

    def makeHotCoffeeTopologyNoOvercome(self):
        self.makeInputTemperatureBump()
        self.makeCoffeeNoOvercomeBump()
        self.makeCokeNoOvercomeBump()
        self.makeSemanticsBinaryCAs()
        self.makeInhibitoryPop()

    def stopFullTopologyFromSpikeSource(self, spikeSource,inhWeight):
        self.fsa.inhibitStateFromSpikeSource(spikeSource,self.semanticsPop, 
                                             self.COFFEE,inhWeight)
        self.fsa.inhibitStateFromSpikeSource(spikeSource,self.semanticsPop, 
                                             self.COKE,inhWeight)
        self.fsa.inhibitStateFromSpikeSource(spikeSource,self.semanticsPop, 
                                             self.COLD,inhWeight)
        self.fsa.inhibitStateFromSpikeSource(spikeSource,self.semanticsPop, 
                                             self.WARM,inhWeight)
        self.fsa.inhibitStateFromSpikeSource(spikeSource,self.semanticsPop, 
                                             self.HOT,inhWeight)
        coffeePop = self.getCoffeeTemperatureBumpNeurons()
        cokePop = self.getCokeTemperatureBumpNeurons()
        for inputTemp in range(0,100): 
            self.fsa.inhibitOneNeuronFromSpikeSource(spikeSource,coffeePop,
                                                       inputTemp,inhWeight)
            self.fsa.inhibitOneNeuronFromSpikeSource(spikeSource,cokePop,
                                                       inputTemp,inhWeight)
            self.fsa.inhibitOneNeuronFromSpikeSource(spikeSource,
                                        self.temperaturePop,inputTemp,inhWeight)

    def printTopologySpikes(self,fileHead):
        self.temperaturePop.write_data(fileHead+"popTemp.pkl")
        self.semanticsPop.write_data(fileHead+"semPop.pkl")
        if ((self.beverageType == BEVERAGE_TYPE_NO_INTERN) or  
            (self.beverageType == BEVERAGE_TYPE_JUST_BUMP)):
            self.coffeeTemperatureObject.write_data(fileHead+"coffeeTemp.pkl")
            self.cokeTemperatureObject.write_data(fileHead+"cokeTemp.pkl")
        else:
            self.coffeeTemperatureObject.printResults(fileHead+"coffeeTemp")
            self.cokeTemperatureObject.printResults(fileHead+"cokeTemp")
        self.inhibitoryPop.write_data(fileHead+"inhibPop.pkl")



