# This is the simple associative memory that we use in the paper. 
#
#
#
#
import pyNN.nest as sim
from nealCoverClass import NealCoverFunctions
from stateMachineClass import FSAHelperFunctions
from hotCoffeeTopologyClass import HotCoffeeClass

#a bit cheeky.  Should get them from hotCoffeeClass
COFFEE = 0
COKE = 1
COLD = 2
WARM = 3
HOT = 4

def createInputsAndStop():
    numberExciteSpikeSources = 196
    numberEpochs = numberExciteSpikeSources
    startSpikeTime = 25
    epochLength = 300
    stopSpikeTimes = []
    startSpikeTimesMatrix = []
    for epoch in range (0,numberExciteSpikeSources):
        stopSpikeTime = (epoch + 1)*epochLength
        stopSpikeTimes = stopSpikeTimes + [stopSpikeTime]
        newStartTime = startSpikeTime + (epoch*epochLength)
        startSpikeTimes = [newStartTime]
        startSpikeTimesMatrix = startSpikeTimesMatrix + [startSpikeTimes]
        
    #print startSpikeTimesMatrix
    startSpikeGens = sim.Population(numberEpochs,sim.SpikeSourceArray,
                {'spike_times': startSpikeTimesMatrix},label="startSpikes")

    #print stopSpikeTimes
    stopSpikeGen = sim.Population(1,sim.SpikeSourceArray,{'spike_times' : stopSpikeTimes},label="stopSpikes")

    return [startSpikeGens, stopSpikeGen]



#--main
#Make weights for plastic synapses 0.01 because that's what we're going
#to change.
#Present an example of all 6 types of triples along with their full range of
#temperatures.


Duration = 60200 

simName = "nest"
spinnVersion = -1
neal = NealCoverFunctions(simName,sim,spinnVersion)
fsa = FSAHelperFunctions(simName,sim,neal,spinnVersion)
hotCoffeeTopology = HotCoffeeClass(simName,sim,neal,fsa,spinnVersion)

hotCoffeeTopology.makeHotCoffeeTopology()
hotCoffeeTopology.buildHotCoffeeTopology()
hotCoffeeTopology.connectTempsToSemanticBeverages()

pop = hotCoffeeTopology.getSemanticsNeurons()
inputPop = hotCoffeeTopology.getTemperatureBumpNeurons()

#TEST 
spikeSources = createInputsAndStop()
stopSpikeSource = spikeSources[1]
startSpikeSources = spikeSources[0]

inhWeight = 2.5

hotCoffeeTopology.stopFullTopologyFromSpikeSource(stopSpikeSource,inhWeight)

for temperature in range(0,25):
    sourceNum = temperature
    fsa.turnOnStateFromMultiSpikeSources(startSpikeSources,sourceNum,pop,COFFEE)
    fsa.turnOnStateFromMultiSpikeSources(startSpikeSources,sourceNum,pop, COLD)
    for inputTemp in range(temperature,temperature+3): 
	fsa.turnOnOneNeuronFromMultiSpikeSource(startSpikeSources,sourceNum,
                                                inputPop,inputTemp)
for temperature in range(25,70):
    sourceNum = temperature
    fsa.turnOnStateFromMultiSpikeSources(startSpikeSources,sourceNum,pop,COFFEE)
    fsa.turnOnStateFromMultiSpikeSources(startSpikeSources,sourceNum,pop, WARM)
    for inputTemp in range(temperature,temperature+3): 
	fsa.turnOnOneNeuronFromMultiSpikeSource(startSpikeSources,sourceNum,
                                                inputPop,inputTemp)

for temperature in range(70,98):
    sourceNum = temperature
    fsa.turnOnStateFromMultiSpikeSources(startSpikeSources,sourceNum,pop,COFFEE)
    fsa.turnOnStateFromMultiSpikeSources(startSpikeSources,sourceNum,pop, HOT)
    for inputTemp in range(temperature,temperature+3): 
	fsa.turnOnOneNeuronFromMultiSpikeSource(startSpikeSources,sourceNum,
                                                inputPop,inputTemp)

for temperature in range(0,10):
    sourceNum = temperature+98
    fsa.turnOnStateFromMultiSpikeSources(startSpikeSources,sourceNum,pop,COKE)
    fsa.turnOnStateFromMultiSpikeSources(startSpikeSources,sourceNum,pop, COLD)
    for inputTemp in range(temperature,temperature+3): 
	fsa.turnOnOneNeuronFromMultiSpikeSource(startSpikeSources,sourceNum,
                                                inputPop,inputTemp)
for temperature in range(10,25):
    sourceNum = temperature+98
    fsa.turnOnStateFromMultiSpikeSources(startSpikeSources,sourceNum,pop,COKE)
    fsa.turnOnStateFromMultiSpikeSources(startSpikeSources,sourceNum,pop, WARM)
    for inputTemp in range(temperature,temperature+3): 
	fsa.turnOnOneNeuronFromMultiSpikeSource(startSpikeSources,sourceNum,
                                                inputPop,inputTemp)

for temperature in range(25,98):
    sourceNum = temperature+98
    fsa.turnOnStateFromMultiSpikeSources(startSpikeSources,sourceNum,pop,COKE)
    fsa.turnOnStateFromMultiSpikeSources(startSpikeSources,sourceNum,pop, HOT)
    for inputTemp in range(temperature,temperature+3): 
	fsa.turnOnOneNeuronFromMultiSpikeSource(startSpikeSources,sourceNum,
                                                inputPop,inputTemp)

neal.nealApplyProjections()
sim.run(Duration)
sim.end()

hotCoffeeTopology.printTopologySpikes("outputs/chris100-")
