# Tests one instance of each category  (e.g. a hot coffee, and coffee 85).
# It reads the temperature associations. 
#

import sys

import pyNN.nest as sim
from nealCoverClass import NealCoverFunctions
from stateMachineClass import FSAHelperFunctions
from hotCoffeeTopologyClass import HotCoffeeClass

#a bit cheeky.  Should get them from hotCoffeeClass
COFFEE = 0
COKE = 1
COLD = 2
WARM = 3
HOT = 4

def createOneInput():
    inputSpikeTimes0 = [25.0] #on
	    
    spikeArray0 = {'spike_times': [inputSpikeTimes0]}
    spikeGen0=sim.Population(1,sim.SpikeSourceArray,spikeArray0,
					   label='inputSpikes_0')
	    
    return spikeGen0

#--main

args = sys.argv
numberArgs = args.__len__()
if (numberArgs == 2):
    testType  = int(args[1])
else:
    testType = 12
print testType

Duration = 5500 

simName = "nest"
spinnVersion = -1
neal = NealCoverFunctions(simName,sim,spinnVersion)
fsa = FSAHelperFunctions(simName,sim,neal,spinnVersion)
hotCoffeeTopology = HotCoffeeClass(simName,sim,neal,fsa,spinnVersion)

hotCoffeeTopology.makeHotCoffeeTopology()
hotCoffeeTopology.buildHotCoffeeTopology()
hotCoffeeTopology.connectTempsToSemanticBeverages()
hotCoffeeTopology.readSemanticTempsToTemps()

pop = hotCoffeeTopology.getSemanticsNeurons()
inputPop = hotCoffeeTopology.getTemperatureBumpNeurons()

#TEST 
spikeSource = createOneInput()
inhWeight = 2.5

if testType == 1: # (coffee)->  just coffeee 15
    fsa.turnOnStateFromSpikeSource(spikeSource,pop, COFFEE)
    for inputTemp in range(15,18): #cold range(0,24)
	fsa.turnOnOneNeuronFromSpikeSource(spikeSource,inputPop,inputTemp)

elif testType == 2: # cold temp ->  coffee and coke
    fsa.turnOnStateFromSpikeSource(spikeSource,pop, COLD)
    for inputTemp in range(15,18): #cold range(0,24)
	fsa.turnOnOneNeuronFromSpikeSource(spikeSource,inputPop,inputTemp)

elif testType == 3: # cold coffee ->  temp
    fsa.turnOnStateFromSpikeSource(spikeSource,pop, COFFEE)
    fsa.turnOnStateFromSpikeSource(spikeSource,pop, COLD)


elif testType == 4: # 
    fsa.turnOnStateFromSpikeSource(spikeSource,pop,COFFEE)
    for inputTemp in range(40,43): #warm coffee range(25,69)
	fsa.turnOnOneNeuronFromSpikeSource(spikeSource,inputPop,inputTemp)
elif testType == 5: # 
    fsa.turnOnStateFromSpikeSource(spikeSource,pop,WARM)
    for inputTemp in range(40,43): #warm coffee range(25,69)
	fsa.turnOnOneNeuronFromSpikeSource(spikeSource,inputPop,inputTemp)
elif testType == 6: # 
    fsa.turnOnStateFromSpikeSource(spikeSource,pop,COFFEE)
    fsa.turnOnStateFromSpikeSource(spikeSource,pop,WARM)

elif testType == 7: # 
    fsa.turnOnStateFromSpikeSource(spikeSource,pop,COFFEE)
    for inputTemp in range(80,83): #hot coffee range(70,99)
	fsa.turnOnOneNeuronFromSpikeSource(spikeSource,inputPop,inputTemp)
elif testType == 8: # 
    fsa.turnOnStateFromSpikeSource(spikeSource,pop,HOT)
    for inputTemp in range(80,83): #hot coffee range(70,99)
	fsa.turnOnOneNeuronFromSpikeSource(spikeSource,inputPop,inputTemp)

elif testType == 9: # 
    fsa.turnOnStateFromSpikeSource(spikeSource,pop,COFFEE)
    fsa.turnOnStateFromSpikeSource(spikeSource,pop,HOT)

elif testType == 10: # 
    fsa.turnOnStateFromSpikeSource(spikeSource,pop,COKE)
    for inputTemp in range(4,7): #cold coke range(0,9)
	fsa.turnOnOneNeuronFromSpikeSource(spikeSource,inputPop,inputTemp)
elif testType == 11: # 
    fsa.turnOnStateFromSpikeSource(spikeSource,pop,COLD)
    for inputTemp in range(4,7): #cold coke range(0,9)
	fsa.turnOnOneNeuronFromSpikeSource(spikeSource,inputPop,inputTemp)
elif testType == 12: # 
    fsa.turnOnStateFromSpikeSource(spikeSource,pop,COKE)
    fsa.turnOnStateFromSpikeSource(spikeSource,pop,COLD)

elif testType == 13: # 
    fsa.turnOnStateFromSpikeSource(spikeSource,pop,COKE)
    for inputTemp in range(15,18): #warm coke range(10,24)
	fsa.turnOnOneNeuronFromSpikeSource(spikeSource,inputPop,inputTemp)
elif testType == 14: # 
    fsa.turnOnStateFromSpikeSource(spikeSource,pop,WARM)
    for inputTemp in range(15,18): #warm coke range(10,24)
	fsa.turnOnOneNeuronFromSpikeSource(spikeSource,inputPop,inputTemp)
elif testType == 15: # 
    fsa.turnOnStateFromSpikeSource(spikeSource,pop,COKE)
    fsa.turnOnStateFromSpikeSource(spikeSource,pop,WARM)

elif testType == 16: # 
    fsa.turnOnStateFromSpikeSource(spikeSource,pop,COKE)
    for inputTemp in range(40,43): #hot coke range(25,99)
	fsa.turnOnOneNeuronFromSpikeSource(spikeSource,inputPop,inputTemp)
elif testType == 17: # 
    fsa.turnOnStateFromSpikeSource(spikeSource,pop,HOT)
    for inputTemp in range(40,43): #hot coke range(25,99)
	fsa.turnOnOneNeuronFromSpikeSource(spikeSource,inputPop,inputTemp)
elif testType == 18: # 
    fsa.turnOnStateFromSpikeSource(spikeSource,pop,COKE)
    fsa.turnOnStateFromSpikeSource(spikeSource,pop,HOT)

neal.nealApplyProjections()
sim.run(Duration)
sim.end()

hotCoffeeTopology.printTopologySpikes("outputs/oneOption")
