# Each of the six semantic pairs has a temperature as a centroid.
# Instances of triples randomly around the centroid are presented.
# We use hot coffee topology with no overcome subnets on 
# beverage bump/merge.

import pyNN.nest as sim
from nealCoverClass import NealCoverFunctions
from stateMachineClass import FSAHelperFunctions
from hotCoffeeTopologyClass import HotCoffeeClass
import random as Random

#a bit cheeky.  Should get them from hotCoffeeClass
COFFEE = 0
COKE = 1
COLD = 2
WARM = 3
HOT = 4

def createInputsAndStop():
    numberExciteSpikeSources = 196
    numberEpochs = numberExciteSpikeSources
    startSpikeTime = 25
    epochLength = 300
    stopSpikeTimes = []
    startSpikeTimesMatrix = []
    for epoch in range (0,numberExciteSpikeSources):
        stopSpikeTime = (epoch + 1)*epochLength
        stopSpikeTimes = stopSpikeTimes + [stopSpikeTime]
        newStartTime = startSpikeTime + (epoch*epochLength)
        startSpikeTimes = [newStartTime]
        startSpikeTimesMatrix = startSpikeTimesMatrix + [startSpikeTimes]
        
    #print startSpikeTimesMatrix
    startSpikeGens = sim.Population(numberEpochs,sim.SpikeSourceArray,
                {'spike_times': startSpikeTimesMatrix},label="startSpikes")

    #print stopSpikeTimes
    stopSpikeGen = sim.Population(1,sim.SpikeSourceArray,{'spike_times' : stopSpikeTimes},label="stopSpikes")

    return [startSpikeGens, stopSpikeGen]


def getGaussTemp(median,oneStdDev):
    temperature = -1
    while (temperature < 0) or (temperature > 97):
        floatTemperature = Random.gauss(median,oneStdDev)
        temperature = int(round(floatTemperature))

    return temperature


#--main
#Make weights for plastic synapses 0.01 because that's what we're going
#to change.
#Present an example of all 6 types of triples along with their full range of
#temperatures.


Duration = 60200 

simName = "nest"
spinnVersion = -1
neal = NealCoverFunctions(simName,sim,spinnVersion)
fsa = FSAHelperFunctions(simName,sim,neal,spinnVersion)
hotCoffeeTopology = HotCoffeeClass(simName,sim,neal,fsa,spinnVersion)
Random.seed(1)

hotCoffeeTopology.makeHotCoffeeTopologyNoOvercome()
hotCoffeeTopology.buildHotCoffeeTopology()
hotCoffeeTopology.connectTempsToSemanticBeverages()
hotCoffeeTopology.readSemanticTempsToTemps()

pop = hotCoffeeTopology.getSemanticsNeurons()
inputPop = hotCoffeeTopology.getTemperatureBumpNeurons()

#TEST 
spikeSources = createInputsAndStop()
stopSpikeSource = spikeSources[1]
startSpikeSources = spikeSources[0]

inhWeight = 2.5

hotCoffeeTopology.stopFullTopologyFromSpikeSource(stopSpikeSource,inhWeight)

for sourceNum in range(0,25):
    temperature = getGaussTemp(12,3)
    fsa.turnOnStateFromMultiSpikeSources(startSpikeSources,sourceNum,pop,COFFEE)
    fsa.turnOnStateFromMultiSpikeSources(startSpikeSources,sourceNum,pop, COLD)
    for inputTemp in range(temperature,temperature+3): 
	fsa.turnOnOneNeuronFromMultiSpikeSource(startSpikeSources,sourceNum,
                                                inputPop,inputTemp)
for sourceNum in range(25,70):
    temperature = getGaussTemp(47,3)
    fsa.turnOnStateFromMultiSpikeSources(startSpikeSources,sourceNum,pop,COFFEE)
    fsa.turnOnStateFromMultiSpikeSources(startSpikeSources,sourceNum,pop, WARM)
    for inputTemp in range(temperature,temperature+3): 
	fsa.turnOnOneNeuronFromMultiSpikeSource(startSpikeSources,sourceNum,
                                                inputPop,inputTemp)

for sourceNum in range(70,98):
    temperature = getGaussTemp(84,3)
    fsa.turnOnStateFromMultiSpikeSources(startSpikeSources,sourceNum,pop,COFFEE)
    fsa.turnOnStateFromMultiSpikeSources(startSpikeSources,sourceNum,pop, HOT)
    for inputTemp in range(temperature,temperature+3): 
	fsa.turnOnOneNeuronFromMultiSpikeSource(startSpikeSources,sourceNum,
                                                inputPop,inputTemp)

for sourceNum in range(98,108):
    temperature = getGaussTemp(3,3)
    fsa.turnOnStateFromMultiSpikeSources(startSpikeSources,sourceNum,pop,COKE)
    fsa.turnOnStateFromMultiSpikeSources(startSpikeSources,sourceNum,pop, COLD)
    for inputTemp in range(temperature,temperature+3): 
	fsa.turnOnOneNeuronFromMultiSpikeSource(startSpikeSources,sourceNum,
                                                inputPop,inputTemp)
for sourceNum in range(108,123):
    temperature = getGaussTemp(17,3)
    fsa.turnOnStateFromMultiSpikeSources(startSpikeSources,sourceNum,pop,COKE)
    fsa.turnOnStateFromMultiSpikeSources(startSpikeSources,sourceNum,pop, WARM)
    for inputTemp in range(temperature,temperature+3): 
	fsa.turnOnOneNeuronFromMultiSpikeSource(startSpikeSources,sourceNum,
                                                inputPop,inputTemp)

for sourceNum in range(123,196):
    temperature = getGaussTemp(62,3)
    fsa.turnOnStateFromMultiSpikeSources(startSpikeSources,sourceNum,pop,COKE)
    fsa.turnOnStateFromMultiSpikeSources(startSpikeSources,sourceNum,pop, HOT)
    for inputTemp in range(temperature,temperature+3): 
	fsa.turnOnOneNeuronFromMultiSpikeSource(startSpikeSources,sourceNum,
                                                inputPop,inputTemp)

neal.nealApplyProjections()
sim.run(Duration)
sim.end()

hotCoffeeTopology.printTopologySpikes("outputs/random-")
