# This reads in the learned weights.
# It tries with each of the 98 temperatures and each 
# of the five semantic primitives.  That's 490 epochs.
#

import pyNN.nest as sim
from nealCoverClass import NealCoverFunctions
from stateMachineClass import FSAHelperFunctions
from hotCoffeeTopologyClass import HotCoffeeClass

#a bit cheeky.  Should get them from hotCoffeeClass
COFFEE = 0
COKE = 1
COLD = 2
WARM = 3
HOT = 4

def createInputs():
    numberExciteSpikeSources = 6
    numberEpochs = numberExciteSpikeSources
    startSpikeTime = 25
    epochLength = 300
    startSpikeTimesMatrix = []
    for epoch in range (0,numberExciteSpikeSources):
        newStartTime = startSpikeTime + (epoch*epochLength)
        startSpikeTimes = [newStartTime]
        startSpikeTimesMatrix = startSpikeTimesMatrix + [startSpikeTimes]
        
    #print startSpikeTimesMatrix
    startSpikeGens = sim.Population(numberEpochs,sim.SpikeSourceArray,
                {'spike_times': startSpikeTimesMatrix},label="startSpikes")

    return startSpikeGens

def createStops():
    numberExciteSpikeSources = 6
    numberEpochs = numberExciteSpikeSources
    startSpikeTime = 25
    epochLength = 300
    stopSpikeTimes = [300,600,900,1200, 1800]
        
    stopSpikeGen1 = sim.Population(1,sim.SpikeSourceArray,{'spike_times' : stopSpikeTimes},label="stopSpikes")

    stopSpikeGen2 = sim.Population(1,sim.SpikeSourceArray,{'spike_times' : [1200,1205,1210,1215,1220,1225,1230,1235,1240,1250,1260,1270,1280,1290]},label="stopSpikes")


    return [stopSpikeGen1,stopSpikeGen2]

#--main

Duration = 1900 

simName = "nest"
spinnVersion = -1
neal = NealCoverFunctions(simName,sim,spinnVersion)
fsa = FSAHelperFunctions(simName,sim,neal,spinnVersion)
hotCoffeeTopology = HotCoffeeClass(simName,sim,neal,fsa,spinnVersion)

hotCoffeeTopology.makeHotCoffeeTopology()
hotCoffeeTopology.buildHotCoffeeTopology()
hotCoffeeTopology.connectTempsToSemanticBeverages()
hotCoffeeTopology.readSemanticTempsToTemps()

pop = hotCoffeeTopology.getSemanticsNeurons()
inputPop = hotCoffeeTopology.getTemperatureBumpNeurons()

#TEST 
startSpikeSources = createInputs()
stopSpikeSource = createStops()

inhWeight = 2.5

hotCoffeeTopology.stopFullTopologyFromSpikeSource(stopSpikeSource[0],inhWeight)

fsa.turnOnStateFromMultiSpikeSources(startSpikeSources,0,pop,HOT)
fsa.turnOnStateFromMultiSpikeSources(startSpikeSources,0,pop,COFFEE)

fsa.turnOnStateFromMultiSpikeSources(startSpikeSources,1,pop,COFFEE)
temperature = 45
for inputTemp in range(temperature,temperature+3): 
    fsa.turnOnOneNeuronFromMultiSpikeSource(startSpikeSources,1,inputPop,
                                            inputTemp)

fsa.turnOnStateFromMultiSpikeSources(startSpikeSources,1,pop,COFFEE)
temperature = 45
for inputTemp in range(temperature,temperature+3): 
    fsa.turnOnOneNeuronFromMultiSpikeSource(startSpikeSources,1,inputPop,
                                            inputTemp)

fsa.turnOnStateFromMultiSpikeSources(startSpikeSources,1,pop,COFFEE)
temperature = 45
for inputTemp in range(temperature,temperature+3): 
    fsa.turnOnOneNeuronFromMultiSpikeSource(startSpikeSources,1,inputPop,
                                            inputTemp)

fsa.turnOnStateFromMultiSpikeSources(startSpikeSources,2,pop,WARM)
temperature = 45
for inputTemp in range(temperature,temperature+3): 
    fsa.turnOnOneNeuronFromMultiSpikeSource(startSpikeSources,2,inputPop,
                                            inputTemp)

fsa.turnOnStateFromMultiSpikeSources(startSpikeSources,3,pop,COFFEE)
temperature = 85
for inputTemp in range(temperature,temperature+3): 
    fsa.turnOnOneNeuronFromMultiSpikeSource(startSpikeSources,3,inputPop,
                                            inputTemp)

fsa.turnOnStateFromMultiSpikeSources(startSpikeSources,4,pop,COFFEE)
fsa.turnOnStateFromMultiSpikeSources(startSpikeSources,4,pop,HOT)


neal.nealApplyProjections()
sim.run(Duration)
sim.end()

hotCoffeeTopology.printTopologySpikes("outputs/small100Out-")
